@@ -29,6 +29,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+#include <unique/uniqueapp.h>
 
 #include "user_share-private.h"
 
@@ -469,6 +470,18 @@ help_button_clicked (GtkButton *button, GtkWidget *window)
 	}
 }
 
+static UniqueResponse
+message_received_cb (UniqueApp         *app,
+		     int                command,
+		     UniqueMessageData *message_data,
+		     guint              time_,
+		     gpointer           user_data)
+{
+  gtk_window_present (GTK_WINDOW (user_data));
+
+  return UNIQUE_RESPONSE_OK;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -487,6 +500,7 @@ main (int argc, char *argv[])
     GtkListStore *store;
     GtkCellRenderer *cell;
     GtkTreeIter iter;
+    UniqueApp *app;
     
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -494,6 +508,13 @@ main (int argc, char *argv[])
     
     gtk_init (&argc, &argv);
 
+    app = unique_app_new ("org.gnome.user-share.properties", NULL);
+    if (unique_app_is_running (app)) {
+      gdk_notify_startup_complete ();
+      unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
+      return 0;
+    }
+
     builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, DATADIR"file-share-properties.ui", &error);
 
@@ -511,6 +532,8 @@ main (int argc, char *argv[])
     window = GTK_WIDGET (gtk_builder_get_object (builder, "user_share_dialog"));
     g_signal_connect (G_OBJECT (window), "delete_event",
 		      G_CALLBACK (gtk_main_quit), NULL);
+    g_signal_connect (app, "message-received",
+		      G_CALLBACK (message_received_cb), window);
 
     client = gconf_client_get_default ();
     gconf_client_add_dir (client,
