@@ -32,6 +32,7 @@
 #include <dbus/dbus-glib.h>
 #include <gconf/gconf-client.h>
 #include <dbus/dbus-glib-lowlevel.h>
+#include <canberra-gtk.h>
 
 #include <string.h>
 
@@ -125,6 +126,7 @@ show_notification (const char *filename)
 {
 	char *file_uri, *notification_text, *display, *mime_type;
 	NotifyNotification *notification;
+	ca_context *ctx;
 	GAppInfo *app;
 
 	file_uri = g_filename_to_uri (filename, NULL, NULL);
@@ -147,17 +149,16 @@ show_notification (const char *filename)
 	mime_type = g_content_type_guess (filename, NULL, 0, NULL);
 	app = g_app_info_get_default_for_type (mime_type, FALSE);
 	if (app != NULL) {
-		/* FIXME duplicate the uri, otherwise libnotify crashes */
-		char *uri;
 		g_object_unref (app);
-		uri = g_strdup (file_uri);
 		notify_notification_add_action (notification, "display", _("Open File"),
 						(NotifyActionCallback) notification_launch_action_on_file_cb,
-						(gpointer) uri, (GFreeFunc) g_free);
+						g_strdup (file_uri), (GFreeFunc) g_free);
 	}
 	notify_notification_add_action (notification, "reveal", _("Reveal File"),
 					(NotifyActionCallback) notification_launch_action_on_file_cb,
-					(gpointer) file_uri, (GFreeFunc) g_free);
+					g_strdup (file_uri), (GFreeFunc) g_free);
+
+	g_free (file_uri);
 	
 	g_signal_connect (G_OBJECT (notification), "closed", G_CALLBACK (on_close_notification), notification);
 
@@ -165,6 +166,13 @@ show_notification (const char *filename)
 		g_warning ("failed to send notification\n");
 	}
 	g_free (notification_text);
+
+	/* Now we do the audio notification */
+	ctx = ca_gtk_context_get ();
+	ca_context_play (ctx, 0,
+			 CA_PROP_EVENT_ID, "complete-download",
+			 CA_PROP_EVENT_DESCRIPTION, "File reception complete",
+			 NULL);
 }
 
 static void
@@ -344,6 +352,7 @@ transfer_started_cb (DBusGProxy *session,
 			return;
 		}
 		g_message ("rejected transfer");
+		g_object_set_data (G_OBJECT (session), "filename", NULL);
 	}
 }
 
@@ -353,16 +362,21 @@ transfer_completed_cb (DBusGProxy *session,
 {
 	GConfClient *client;
 	gboolean display_notify; 
+	const char *filename;
 
-	g_message ("file finish transfer: %s",
-		   (char *) g_object_get_data (G_OBJECT (session), "filename"));
+	filename = (const char *) g_object_get_data (G_OBJECT (session), "filename");
+
+	g_message ("file finish transfer: %s", filename);
+
+	if (filename == NULL)
+		return;
 	
 	client = gconf_client_get_default ();	
 	display_notify = gconf_client_get_bool (client, FILE_SHARING_BLUETOOTH_OBEXPUSH_NOTIFY, NULL);
 	g_object_unref (client);
 	
 	if (display_notify) {
-		show_notification (g_object_get_data (G_OBJECT (session), "filename"));
+		show_notification (filename);
 	} else {
 		hide_statusicon ();
 	}
@@ -376,6 +390,7 @@ cancelled_cb (DBusGProxy *session,
 	//FIXME implement properly, we never actually finished the transfer
 	g_message ("transfered was cancelled by the sender");
 	transfer_completed_cb (session, user_data);
+	hide_statusicon ();
 }
 
 static void
@@ -459,6 +474,13 @@ obexpush_up (void)
 	if (dbus_g_proxy_call (server_proxy, "Start", &err,
 			   G_TYPE_STRING, download_dir, G_TYPE_BOOLEAN, TRUE, G_TYPE_BOOLEAN, FALSE, G_TYPE_INVALID,
 			   G_TYPE_INVALID) == FALSE) {
+		if (g_error_matches (err, DBUS_GERROR, DBUS_GERROR_REMOTE_EXCEPTION) != FALSE &&
+		    dbus_g_error_has_name (err, "org.openobex.Error.Started") != FALSE) {
+		    	g_error_free (err);
+		    	g_message ("already started, ignoring error");
+		    	g_free (download_dir);
+		    	return;
+		}
 		g_printerr ("Starting Bluetooth ObexPush server failed: %s\n",
 			    err->message);
 		g_error_free (err);
@@ -482,12 +504,8 @@ obexpush_stop (gboolean stop_manager)
 		return;
 
 	if (dbus_g_proxy_call (server_proxy, "Close", &err, G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
-		const gchar *error_name = NULL;
-
-		if (err != NULL && err->code == DBUS_GERROR_REMOTE_EXCEPTION)
-			error_name = dbus_g_error_get_name (err);
-		if (error_name == NULL ||
-		    (error_name != NULL && strcmp (error_name, "org.openobex.Error.NotStarted") != 0)) {
+		if (g_error_matches (err, DBUS_GERROR, DBUS_GERROR_REMOTE_EXCEPTION) == FALSE ||
+		    dbus_g_error_has_name (err, "org.openobex.Error.NotStarted") == FALSE) {
 			g_printerr ("Stopping Bluetooth ObexPush server failed: %s\n",
 				    err->message);
 			g_error_free (err);
@@ -502,6 +520,8 @@ obexpush_stop (gboolean stop_manager)
 		g_object_unref (manager_proxy);
 		manager_proxy = NULL;
 	}
+
+	//FIXME stop all the notifications
 }
 
 void
